import axios, { AxiosRequestConfig, Method } from 'axios';
import { store } from '../store';
import { refresh } from '../controllers';

export const request = (
  method: Method,
  url: string,
  data: {} | null = null,
  headers: {} | null = null
) => {
  const API = axios.create({ baseURL: process.env.REACT_APP_BASE_URL });

  API.interceptors.request.use(
    async config => {
      const {
        auth: {
          token,
        },
      } = store.getState();

      const expiryDate = new Date();

      expiryDate.setMinutes(
        expiryDate.getMinutes() + (Number(process.env.REACT_APP_TTL) - 1)
      );

      if (expiryDate.getTime() < Date.now()) {
        await store.dispatch(refresh());

        if (config.headers) config.headers.authorization = `Bearer ${token}`;
      }

      return config;
    },
    error => {
      return Promise.reject(error);
    },
  );

  return API.request({ method, url, data, headers } as AxiosRequestConfig);
};
